@@ -1,4 +1,4 @@
-//===------ polly/CodeGeneration.h - The Polly code generator *- C++ -*-===//
+//===- polly/CodeGeneration.h - The Polly code generator --------*- C++ -*-===//
 //
 //                     The LLVM Compiler Infrastructure
 //
@@ -6,18 +6,25 @@
 // License. See LICENSE.TXT for details.
 //
 //===----------------------------------------------------------------------===//
-//
-//===----------------------------------------------------------------------===//
 
 #ifndef POLLY_CODEGENERATION_H
 #define POLLY_CODEGENERATION_H
 
+#include "IRBuilder.h"
 #include "polly/Config/config.h"
 #include "polly/ScopPass.h"
-#include "isl/map.h"
-#include "isl/set.h"
+#include "llvm/IR/PassManager.h"
+
+namespace llvm {
+
+class BasicBlock;
+
+} // namespace llvm
 
 namespace polly {
+
+class Scop;
+
 enum VectorizerChoice {
   VECTORIZER_NONE,
   VECTORIZER_STRIPMINE,
@@ -25,10 +32,19 @@ enum VectorizerChoice {
 };
 extern VectorizerChoice PollyVectorizerChoice;
 
+/// Mark a basic block unreachable.
+///
+/// Marks the basic block @p Block unreachable by equipping it with an
+/// UnreachableInst.
+void markBlockUnreachable(BasicBlock &Block, PollyIRBuilder &Builder);
+
 struct CodeGenerationPass : public PassInfoMixin<CodeGenerationPass> {
   PreservedAnalyses run(Scop &S, ScopAnalysisManager &SAM,
                         ScopStandardAnalysisResults &AR, SPMUpdater &U);
 };
+
+extern bool PerfMonitoring;
+
 } // namespace polly
 
 #endif // POLLY_CODEGENERATION_H
