@@ -1,4 +1,4 @@
-//=== ScopDetectionDiagnostic.h -- Diagnostic for ScopDetection -*- C++ -*-===//
+//===- ScopDetectionDiagnostic.h - Diagnostic for ScopDetection -*- C++ -*-===//
 //
 //                     The LLVM Compiler Infrastructure
 //
@@ -17,29 +17,32 @@
 // to diagnose the error and generate a helpful error message.
 //
 //===----------------------------------------------------------------------===//
-#ifndef POLLY_SCOP_DETECTION_DIAGNOSTIC_H
-#define POLLY_SCOP_DETECTION_DIAGNOSTIC_H
+
+#ifndef POLLY_SCOPDETECTIONDIAGNOSTIC_H
+#define POLLY_SCOPDETECTIONDIAGNOSTIC_H
 
 #include "llvm/ADT/SmallVector.h"
-#include "llvm/ADT/Statistic.h"
-#include "llvm/ADT/Twine.h"
-#include "llvm/Analysis/AliasSetTracker.h"
 #include "llvm/Analysis/LoopInfo.h"
-#include "llvm/Analysis/OptimizationDiagnosticInfo.h"
-#include "llvm/Analysis/ScalarEvolutionExpressions.h"
-#include "llvm/IR/BasicBlock.h"
-#include "llvm/IR/Value.h"
-#include "llvm/Support/Casting.h"
+#include "llvm/IR/DebugLoc.h"
+#include "llvm/IR/Instruction.h"
+#include <cstddef>
 #include <memory>
 #include <string>
+#include <utility>
+#include <vector>
 
 using namespace llvm;
 
 namespace llvm {
-class SCEV;
+
+class AliasSet;
 class BasicBlock;
-class Value;
+class OptimizationRemarkEmitter;
+class raw_ostream;
 class Region;
+class SCEV;
+class Value;
+
 } // namespace llvm
 
 namespace polly {
@@ -54,6 +57,7 @@ BBPair getBBPairForRegion(const Region *R);
 void getDebugLocations(const BBPair &P, DebugLoc &Begin, DebugLoc &End);
 
 class RejectLog;
+
 /// Emit optimization remarks about the rejected regions to the user.
 ///
 /// This emits the content of the reject log as optimization remarks.
@@ -111,7 +115,6 @@ enum class RejectReasonKind {
 /// diagnostic information to help clients figure out what and where something
 /// went wrong in the Scop detection.
 class RejectReason {
-  //===--------------------------------------------------------------------===//
 private:
   const RejectReasonKind Kind;
 
@@ -119,11 +122,11 @@ protected:
   static const DebugLoc Unknown;
 
 public:
-  RejectReasonKind getKind() const { return Kind; }
-
   RejectReason(RejectReasonKind K);
 
-  virtual ~RejectReason() {}
+  virtual ~RejectReason() = default;
+
+  RejectReasonKind getKind() const { return Kind; }
 
   /// Generate the remark name to identify this remark.
   ///
@@ -153,20 +156,20 @@ public:
   /// Get the source location of this error.
   ///
   /// @return The debug location for this error.
-  virtual const llvm::DebugLoc &getDebugLoc() const;
+  virtual const DebugLoc &getDebugLoc() const;
 };
 
-typedef std::shared_ptr<RejectReason> RejectReasonPtr;
+using RejectReasonPtr = std::shared_ptr<RejectReason>;
 
 /// Stores all errors that occurred during the detection.
 class RejectLog {
   Region *R;
-  llvm::SmallVector<RejectReasonPtr, 1> ErrorReports;
+  SmallVector<RejectReasonPtr, 1> ErrorReports;
 
 public:
   explicit RejectLog(Region *R) : R(R) {}
 
-  typedef llvm::SmallVector<RejectReasonPtr, 1>::const_iterator iterator;
+  using iterator = SmallVector<RejectReasonPtr, 1>::const_iterator;
 
   iterator begin() const { return ErrorReports.begin(); }
   iterator end() const { return ErrorReports.end(); }
@@ -189,7 +192,6 @@ public:
 /// Scop candidates that violate structural restrictions can be grouped under
 /// this reject reason class.
 class ReportCFG : public RejectReason {
-  //===--------------------------------------------------------------------===//
 public:
   ReportCFG(const RejectReasonKind K);
 
@@ -215,10 +217,10 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
@@ -239,11 +241,11 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
@@ -265,11 +267,11 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
@@ -279,8 +281,6 @@ public:
 /// Scop candidates that violate restrictions to affinity are reported under
 /// this class.
 class ReportAffFunc : public RejectReason {
-  //===--------------------------------------------------------------------===//
-
 protected:
   // The instruction that caused non-affinity to occur.
   const Instruction *Inst;
@@ -295,17 +295,13 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual const DebugLoc &getDebugLoc() const override {
-    return Inst->getDebugLoc();
-  }
+  const DebugLoc &getDebugLoc() const override { return Inst->getDebugLoc(); }
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures a condition that is based on an 'undef' value.
 class ReportUndefCond : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The BasicBlock we found the broken condition in.
   BasicBlock *BB;
 
@@ -320,9 +316,9 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
   //@}
 };
 
@@ -331,8 +327,6 @@ public:
 ///
 /// Conditions have to be either constants or icmp instructions.
 class ReportInvalidCond : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The BasicBlock we found the broken condition in.
   BasicBlock *BB;
 
@@ -347,17 +341,15 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures an undefined operand.
 class ReportUndefOperand : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The BasicBlock we found the undefined operand in.
   BasicBlock *BB;
 
@@ -372,17 +364,15 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures a non-affine branch.
 class ReportNonAffBranch : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The BasicBlock we found the non-affine branch in.
   BasicBlock *BB;
 
@@ -408,16 +398,15 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures a missing base pointer.
 class ReportNoBasePtr : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
 public:
   ReportNoBasePtr(const Instruction *Inst)
       : ReportAffFunc(RejectReasonKind::NoBasePtr, Inst) {}
@@ -429,16 +418,15 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures an undefined base pointer.
 class ReportUndefBasePtr : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
 public:
   ReportUndefBasePtr(const Instruction *Inst)
       : ReportAffFunc(RejectReasonKind::UndefBasePtr, Inst) {}
@@ -450,17 +438,15 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures a base pointer that is not invariant in the region.
 class ReportVariantBasePtr : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The variant base pointer.
   Value *BaseValue;
 
@@ -476,18 +462,16 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures a non-affine access function.
 class ReportNonAffineAccess : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The non-affine access function.
   const SCEV *AccessFunction;
 
@@ -509,18 +493,16 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Report array accesses with differing element size.
 class ReportDifferentArrayElementSize : public ReportAffFunc {
-  //===--------------------------------------------------------------------===//
-
   // The base pointer of the memory access.
   const Value *BaseValue;
 
@@ -536,18 +518,16 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with non affine loop bounds.
 class ReportLoopBound : public RejectReason {
-  //===--------------------------------------------------------------------===//
-
   // The offending loop.
   Loop *L;
 
@@ -569,19 +549,17 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors when loop has no exit.
 class ReportLoopHasNoExit : public RejectReason {
-  //===--------------------------------------------------------------------===//
-
   /// The loop that has no exit.
   Loop *L;
 
@@ -599,19 +577,17 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors when not all loop latches are part of the scop.
 class ReportLoopOnlySomeLatches : public RejectReason {
-  //===--------------------------------------------------------------------===//
-
   /// The loop for which not all loop latches are part of the scop.
   Loop *L;
 
@@ -629,19 +605,17 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with non-side-effect-known function calls.
 class ReportFuncCall : public RejectReason {
-  //===--------------------------------------------------------------------===//
-
   // The offending call instruction.
   Instruction *Inst;
 
@@ -655,20 +629,19 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with aliasing.
 class ReportAlias : public RejectReason {
-  //===--------------------------------------------------------------------===//
 public:
-  typedef std::vector<const llvm::Value *> PointerSnapshotTy;
+  using PointerSnapshotTy = std::vector<const Value *>;
 
 private:
   /// Format an invalid alias set.
@@ -695,18 +668,17 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Base class for otherwise ungrouped reject reasons.
 class ReportOther : public RejectReason {
-  //===--------------------------------------------------------------------===//
 public:
   ReportOther(const RejectReasonKind K);
 
@@ -717,16 +689,14 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual std::string getMessage() const override;
+  std::string getRemarkName() const override;
+  std::string getMessage() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with bad IntToPtr instructions.
 class ReportIntToPtr : public ReportOther {
-  //===--------------------------------------------------------------------===//
-
   // The offending base value.
   Instruction *BaseValue;
 
@@ -740,17 +710,16 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with alloca instructions.
 class ReportAlloca : public ReportOther {
-  //===--------------------------------------------------------------------===//
   Instruction *Inst;
 
 public:
@@ -763,17 +732,16 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with unknown instructions.
 class ReportUnknownInst : public ReportOther {
-  //===--------------------------------------------------------------------===//
   Instruction *Inst;
 
 public:
@@ -786,17 +754,16 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with regions containing the function entry block.
 class ReportEntry : public ReportOther {
-  //===--------------------------------------------------------------------===//
   BasicBlock *BB;
 
 public:
@@ -809,18 +776,17 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Report regions that seem not profitable to be optimized.
 class ReportUnprofitable : public ReportOther {
-  //===--------------------------------------------------------------------===//
   Region *R;
 
 public:
@@ -833,19 +799,17 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual std::string getEndUserMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  std::string getEndUserMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
   //@}
 };
 
 //===----------------------------------------------------------------------===//
 /// Captures errors with non-simple memory accesses.
 class ReportNonSimpleMemoryAccess : public ReportOther {
-  //===--------------------------------------------------------------------===//
-
   // The offending call instruction.
   Instruction *Inst;
 
@@ -859,14 +823,14 @@ public:
 
   /// @name RejectReason interface
   //@{
-  virtual std::string getRemarkName() const override;
-  virtual const Value *getRemarkBB() const override;
-  virtual std::string getMessage() const override;
-  virtual const DebugLoc &getDebugLoc() const override;
-  virtual std::string getEndUserMessage() const override;
+  std::string getRemarkName() const override;
+  const Value *getRemarkBB() const override;
+  std::string getMessage() const override;
+  const DebugLoc &getDebugLoc() const override;
+  std::string getEndUserMessage() const override;
   //@}
 };
 
 } // namespace polly
 
-#endif // POLLY_SCOP_DETECTION_DIAGNOSTIC_H
+#endif // POLLY_SCOPDETECTIONDIAGNOSTIC_H
